# Copyright (c) 2017 "Shopify inc." All rights reserved.
# Use of this source code is governed by a MIT-style license that can be found in the LICENSE file.
from __future__ import unicode_literals

# pylint: disable=import-modules-only

from pyoozie.xml import WorkflowBuilder

from pyoozie.client import OozieClient

from pyoozie.exceptions import OozieException

from pyoozie.model import ArtifactType
from pyoozie.model import CoordinatorActionStatus
from pyoozie.model import CoordinatorStatus
from pyoozie.model import WorkflowActionStatus
from pyoozie.model import WorkflowStatus
from pyoozie.model import parse_coordinator_id
from pyoozie.model import parse_workflow_id

from pyoozie.tags import Action
from pyoozie.tags import Configuration
from pyoozie.tags import CoordinatorApp
from pyoozie.tags import Credential
from pyoozie.tags import Decision
from pyoozie.tags import Email
from pyoozie.tags import ExecutionOrder
from pyoozie.tags import EXEC_FIFO
from pyoozie.tags import EXEC_LAST_ONLY
from pyoozie.tags import EXEC_LIFO
from pyoozie.tags import EXEC_NONE
from pyoozie.tags import GlobalConfiguration
from pyoozie.tags import Kill
from pyoozie.tags import Parallel
from pyoozie.tags import Parameters
from pyoozie.tags import Serial
from pyoozie.tags import Shell
from pyoozie.tags import SubWorkflow
from pyoozie.tags import validate_xml_id
from pyoozie.tags import validate_xml_name
from pyoozie.tags import WorkflowApp


__version__ = '0.0.5'

__all__ = (
    # builder
    'WorkflowBuilder',

    # client
    'OozieClient',

    # exceptions
    'OozieException',

    # model
    'ArtifactType',
    'CoordinatorActionStatus',
    'CoordinatorStatus',
    'WorkflowActionStatus',
    'WorkflowStatus',
    'parse_coordinator_id',
    'parse_workflow_id',

    # tags
    'Action',
    'Configuration',
    'CoordinatorApp',
    'Credential',
    'Decision',
    'Email',
    'ExecutionOrder',
    'EXEC_FIFO',
    'EXEC_LAST_ONLY',
    'EXEC_LIFO',
    'EXEC_NONE',
    'GlobalConfiguration',
    'Kill',
    'Parallel',
    'Parameters',
    'Serial',
    'Shell',
    'SubWorkflow',
    'validate_xml_id',
    'validate_xml_name',
    'WorkflowApp'
)



# Copyright (c) 2017 "Shopify inc." All rights reserved.
# Use of this source code is governed by a MIT-style license that can be found in the LICENSE file.
from __future__ import unicode_literals

import warnings

from pyoozie import tags


def _workflow_submission_xml(username, workflow_xml_path, configuration=None, indent=False):
    """Generate a Workflow XML submission message to POST to Oozie."""
    submission = tags.Configuration(configuration)
    submission.update({
        'user.name': username,
        'oozie.wf.application.path': workflow_xml_path,
    })
    return submission.xml(indent)


def _coordinator_submission_xml(username, coord_xml_path, configuration=None, indent=False):
    """Generate a Coordinator XML submission message to POST to Oozie."""
    submission = tags.Configuration(configuration)
    submission.update({
        'user.name': username,
        'oozie.coord.application.path': coord_xml_path,
    })
    return submission.xml(indent)


class WorkflowBuilder(object):

    def __init__(self, name):
        warnings.warn("WorkflowBuilder will be replaced in the future with a different API to construct workflows",
                      PendingDeprecationWarning)

        # Initially, let's just use a static template and only one action payload and one action on error
        self._name = tags.validate_xml_name(name)
        self._action_name = None
        self._action_payload = None
        self._action_error = None
        self._kill_message = None

    def add_action(self, name, action, action_on_error, kill_on_error='${wf:lastErrorNode()} - ${wf:id()}'):
        # Today you can't rename your action and you can only have one, but in the future you can add multiple
        # named actions
        if any((self._action_name, self._action_payload, self._action_error, self._kill_message)):
            raise NotImplementedError("Can only add one action in this version")
        else:
            self._action_name = tags.validate_xml_id('action-' + name)
            self._action_payload = action
            self._action_error = action_on_error
            self._kill_message = kill_on_error

        return self

    def build(self, indent=False):
        def format_xml(xml):
            xml = xml.decode('utf-8').replace("<?xml version='1.0' encoding='UTF-8'?>", '')
            return '\n'.join([(' ' * 8) + line for line in xml.strip().split('\n')])
        return '''
<?xml version="1.0" encoding="UTF-8"?>
<workflow-app xmlns="uri:oozie:workflow:0.5"
              name="{name}">
    <start to="{action_name}" />
    <action name="{action_name}">
{action_payload_xml}
        <ok to="end" />
        <error to="action-error" />
    </action>
    <action name="action-error">
{action_error_xml}
        <ok to="kill" />
        <error to="kill" />
    </action>
    <kill name="kill">
        <message>{kill_message}</message>
    </kill>
    <end name="end" />
</workflow-app>
'''.format(action_payload_xml=format_xml(self._action_payload.xml(indent=indent)),
           action_error_xml=format_xml(self._action_error.xml(indent=indent)),
           kill_message=self._kill_message,
           action_name=self._action_name,
           name=self._name).strip().encode('utf-8')



#!/usr/bin/env python
# Copyright (c) 2017 "Shopify inc." All rights reserved.
# Use of this source code is governed by a MIT-style license that can be found in the LICENSE file.
import re


try:
    import setuptools as setuplib
except ImportError:
    import distutils.core as setuplib


def get_version():
    version = None
    with open('pyoozie/__init__.py', 'r') as fdesc:
        version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fdesc.read(), re.MULTILINE).group(1)
    if not version:
        raise RuntimeError('Cannot find version information')
    return version


setuplib.setup(
    name='pyoozie',
    version=get_version(),
    description='A Python client for querying and scheduling with Oozie',
    author='Shopify Data Acceleration',
    author_email='data-acceleration@shopify.com',
    url='https://github.com/Shopify/pyoozie',
    packages=['pyoozie'],
    install_requires=[
        'enum34>=0.9.23',
        'requests>=2.12.3',
        'six>=1.10.0',
        'typing',
        'untangle>=1.1.0',
        'yattag>=1.7.2',
    ],
    extras_require={
        'deploy': [
            'twine>=1.9.1',
            'setuptools>=0.9'
        ],
        'test': [
            'autopep8',
            'flake8',
            'mock',
            'pycodestyle == 2.2.0',
            'pylint>=1.7.1,<1.8',
            'pytest-cov>=2.4.0,<2.6',  # pinned, see https://github.com/z4r/python-coveralls/issues/66
            'pytest-randomly',
            'pytest>=3.0',
            'requests-mock',
            'shopify_python==0.4.1',
            'xmltodict',
            'pywebhdfs>=0.4.1',
        ],
        'test: python_version ~= "3.4"': [  # mypy 0.700 dropped python 3.4 support
            'mypy == 0.670',
        ],
        'test: python_version >= "3.5"': [
            'mypy',
        ],
        'docs': [
            'sphinx >= 1.6',
            'sphinx_rtd_theme',
        ],
    },
    license="MIT",
    keywords=['oozie'],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
)



# Copyright (c) 2017 "Shopify inc." All rights reserved.
# Use of this source code is governed by a MIT-style license that can be found in the LICENSE file.
from __future__ import unicode_literals


import requests


class OozieException(Exception):

    def __init__(self, message='', caused_by=None):
        super(OozieException, self).__init__(message or caused_by and str(caused_by))
        self.caused_by = caused_by

    @classmethod
    def coordinator_not_found(cls, artifact_id, caused_by=None):
        message = "Coordinator '{}' not found".format(artifact_id)
        return OozieArtifactNotFoundException(message, caused_by)

    @classmethod
    def coordinator_action_not_found(cls, artifact_id, action, caused_by=None):
        message = "Coordinator action '{}@{}' not found".format(artifact_id, action)
        return OozieArtifactNotFoundException(message, caused_by)

    @classmethod
    def workflow_not_found(cls, artifact_id, caused_by=None):
        message = "Workflow '{}' not found".format(artifact_id)
        return OozieArtifactNotFoundException(message, caused_by)

    @classmethod
    def workflow_action_not_found(cls, artifact_id, action, caused_by=None):
        message = "Workflow action '{}@{}' not found".format(artifact_id, action)
        return OozieArtifactNotFoundException(message, caused_by)

    @classmethod
    def job_not_found(cls, artifact_id, caused_by=None):
        message = "'{}' does not match any known job".format(artifact_id)
        return OozieArtifactNotFoundException(message, caused_by)

    @classmethod
    def operation_failed(cls, operation, caused_by=None):
        message = "Operation failed: {}".format(operation)
        return OozieOperationFailedException(message, caused_by)

    @classmethod
    def parse_error(cls, message, caused_by=None):
        return OozieParsingException(message, caused_by)

    @classmethod
    def required_key_missing(cls, key, artifact, caused_by=None):
        message = "Required key '{}' missing or invalid in {}".format(key, artifact.__class__.__name__)
        return OozieParsingException(message, caused_by)

    @classmethod
    def communication_error(cls, message=None, caused_by=None):
        if not message:
            if caused_by and isinstance(caused_by, requests.RequestException):
                if caused_by.response is not None:
                    message = caused_by.response.headers.get('oozie-error-message', caused_by.response.reason)
        return OozieCommunicationException(message, caused_by)


class OozieArtifactNotFoundException(OozieException):
    pass


class OozieOperationFailedException(OozieException):
    pass


class OozieParsingException(OozieException):
    pass


class OozieCommunicationException(OozieException):
    pass
